@@ -2,6 +2,7 @@ import os
 import glob
 import argparse
 from Bio import SeqIO
+from Bio import AlignIO
 import multiprocessing as mp
 from distutils.spawn import find_executable
 
@@ -12,7 +13,7 @@ Marker2Tree_usage = '''
 Dependencies: blastp, mafft-einsi, trimal, iqtree
 
 # example commands
-BioSAK Marker2Tree -m marker_seq -mx fa -aa gnm_faa_files -aax faa -o op_dir -e 30 -t 6 -pl /home-user/wzsong/Scripts/catfasta2phyml.pl
+BioSAK Marker2Tree -m marker_seq -mx fa -aa gnm_faa_files -aax faa -o op_dir -e 30 -t 6
 
 ========================================================================================
 '''
@@ -411,6 +412,70 @@ def AssessMarkerPA(trimmed_aln_dir, gnm_set, gnm_group_txt, present_pct_cutoff_l
     print('Assessment results exported to:\n%s\n%s' % (assess_summary_1_txt, assess_summary_2_txt))
 
 
+def catfasta2phy(msa_dir, msa_ext, concatenated_msa_phy, partition_file):
+
+    concatenated_msa_fasta = '%s.fasta' % concatenated_msa_phy
+    msa_file_re            = '%s/*.%s'  % (msa_dir, msa_ext)
+    msa_file_list          = [os.path.basename(file_name) for file_name in glob.glob(msa_file_re)]
+    msa_file_list_sorted   = sorted(msa_file_list)
+
+    complete_gnm_set = set()
+    for each_msa_file in msa_file_list:
+        pwd_msa = '%s/%s' % (msa_dir, each_msa_file)
+        for each_seq in SeqIO.parse(pwd_msa, 'fasta'):
+            complete_gnm_set.add(each_seq.id)
+
+    complete_gnm_list_sorted = sorted([i for i in complete_gnm_set])
+
+    # initialize concatenated msa dict
+    gnm_to_seq_dict = {i: '' for i in complete_gnm_list_sorted}
+    msa_len_dict = dict()
+    for each_msa_file in msa_file_list_sorted:
+        gene_id = each_msa_file.split('.' + msa_ext)[0]
+
+        # read in msa
+        current_msa_len = 0
+        current_msa_len_set = set()
+        pwd_current_msa = '%s/%s' % (msa_dir, each_msa_file)
+        current_msa_seq_dict = dict()
+        for each_seq in SeqIO.parse(pwd_current_msa, 'fasta'):
+            complete_gnm_set.add(each_seq.id)
+            current_msa_seq_dict[each_seq.id] = str(each_seq.seq)
+            current_msa_len_set.add(len(each_seq.seq))
+            current_msa_len = len(each_seq.seq)
+
+        if len(current_msa_len_set) != 1:
+            print('Sequences with different length were found in %s, program exited!' % each_msa_file)
+            exit()
+
+        msa_len_dict[gene_id] = current_msa_len
+
+        # add sequence to concatenated msa dict
+        for each_gnm in complete_gnm_list_sorted:
+            msa_seq = current_msa_seq_dict.get(each_gnm, current_msa_len*'-')
+            gnm_to_seq_dict[each_gnm] += msa_seq
+
+    # write out concatenated msa
+    concatenated_msa_handle = open(concatenated_msa_fasta, 'w')
+    for each_gnm in complete_gnm_list_sorted:
+        concatenated_msa_handle.write('>%s\n' % each_gnm)
+        concatenated_msa_handle.write('%s\n' % gnm_to_seq_dict[each_gnm])
+    concatenated_msa_handle.close()
+
+    # write out partition file
+    end_pos = 0
+    partition_file_handle = open(partition_file, 'w')
+    for each_m in msa_file_list_sorted:
+        gene_id = each_m.split('.' + msa_ext)[0]
+        current_m_len = msa_len_dict[gene_id]
+        partition_file_handle.write('%s = %s-%s\n' % (each_m, (end_pos + 1), (end_pos + current_m_len)))
+        end_pos += current_m_len
+    partition_file_handle.close()
+
+    # convert msa in fasta to phy
+    AlignIO.convert(concatenated_msa_fasta, 'fasta', concatenated_msa_phy, 'phylip-relaxed')
+
+
 def Marker2Tree(args):
 
     marker_seq_dir          = args['m']
@@ -422,7 +487,7 @@ def Marker2Tree(args):
     e_value                 = args['e']
     num_of_threads          = args['t']
     force_overwrite         = args['f']
-    catfasta2phyml_pl       = args['pl']
+    #catfasta2phyml_pl       = args['pl']
     js_cpu_num              = args['jst']
     pa_cutoff_str           = args['pac']
     skip_align_trim         = args['skip_align_trim']
@@ -620,9 +685,11 @@ def Marker2Tree(args):
             qualified_cutoff_list.append(each_cutoff)
             pwd_concatenated_marker_phy       = '%s/qualified_marker_PA_%s_concatenated.phy'                % (trimmed_msa_PA_concatenated_dir, each_cutoff)
             pwd_concatenated_marker_partition = '%s/qualified_marker_PA_%s_concatenated_partition.txt'      % (trimmed_msa_PA_concatenated_dir, each_cutoff)
-            catfasta2phyml_cmd                = 'perl %s --sequential --concatenate %s/*.aln > %s 2> %s'    % (catfasta2phyml_pl, current_cutoff_trimmed_msa_dir, pwd_concatenated_marker_phy, pwd_concatenated_marker_partition)
+            #catfasta2phyml_cmd                = 'perl %s --sequential --concatenate %s/*.aln > %s 2> %s'    % (catfasta2phyml_pl, current_cutoff_trimmed_msa_dir, pwd_concatenated_marker_phy, pwd_concatenated_marker_partition)
             #print('running: ' + catfasta2phyml_cmd)
-            os.system(catfasta2phyml_cmd)
+            #os.system(catfasta2phyml_cmd)
+            catfasta2phy(current_cutoff_trimmed_msa_dir, 'aln', pwd_concatenated_marker_phy, pwd_concatenated_marker_partition)
+
 
     ########## get guide tree and C60+PMSF tree for each set of marker set ##########
 
@@ -712,7 +779,6 @@ if __name__ == '__main__':
     parser.add_argument('-mmn',             required=False, default=10, type=int,   help='minimal marker number, default: 10')
     parser.add_argument('-jst',             required=False, default='6',            help='threads to request in job script, for running iqtree')
     parser.add_argument('-qsub',            required=False, action="store_true",    help='submit job scripts')
-    parser.add_argument('-pl',              required=True,                          help='path to catfasta2phyml.pl')
     parser.add_argument('-f',               required=False, action="store_true",    help='force overwrite')
     args = vars(parser.parse_args())
     Marker2Tree(args)
